package com.displaynone.acss.controllers.auth;

import com.displaynone.acss.components.auth.AuthComponent;
import com.displaynone.acss.components.auth.models.AuthTokenPair;
import com.displaynone.acss.components.auth.models.user.UserDTO;
import com.displaynone.acss.components.auth.models.user.UserModel;
import com.displaynone.acss.exception.generics.BadRequestHTTPException;
import com.displaynone.acss.exception.generics.NotFoundHTTPException;
import com.displaynone.acss.exception.generics.UnauthorizedHTTPException;
import com.displaynone.acss.utils.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api/auth")
public class AuthController {
    private final AuthComponent authComponent;

    @Autowired
    public AuthController(AuthComponent authComponent) {
        this.authComponent = authComponent;
    }

    @PostMapping("/login")
    public ResponseEntity<AuthTokenPair> login(@RequestBody LoginRequestBody loginRequest) {
        String login = loginRequest.getLogin();
        if (login == null) throw new BadRequestHTTPException("Login not specified");

        Pair<UserModel, AuthTokenPair> authInfo;

        try {
            authInfo = authComponent.authenticate(login);
        } catch (NotFoundHTTPException e) {
            throw new UnauthorizedHTTPException(e.getMessage());
        }

        return ResponseEntity.ok(authInfo.getSecond());
    }

    @PostMapping("/refresh")
    public ResponseEntity<?> refreshToken(@RequestBody TokenRefreshRequestBody request) {
        String requestRefreshToken = request.getRefreshToken();
        if (requestRefreshToken == null) throw new BadRequestHTTPException("Refresh token not specified");

        AuthTokenPair authTokenPair;
        try {
            authTokenPair = authComponent.refresh(requestRefreshToken);
        } catch (NotFoundHTTPException e) {
            throw new UnauthorizedHTTPException(e.getMessage());
        }

        return ResponseEntity.ok(authTokenPair);
    }
}